/* ======================================================================
     "libbasIOstr.h"
       - header file for "libbasIOstr.cc"
             Copyright (c) 2002-2012  by A. Miyoshi, Univ. Tokyo
 ====================================================================== */

#ifndef _LIBBASIOSTR_H_
#define _LIBBASIOSTR_H_

#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <vector>
#include <set>
#include <map>
#include <cctype>
#include <cstring>
#include <cstdlib>
#include <cmath>
#include <complex>
using namespace std;

// ----- system constants

#ifndef true
#define true (-1)
#endif
#ifndef false
#define false 0
#endif
#ifndef M_PI
#define M_PI    3.14159265358979323846
#endif
#define BEX1DREV "BEx1D rev. 2012.03.12"
#define NORM 0
#define ERRE 1
#define VAL0THR 1e-50              // threshold for zero value

// ----- physical constants and conversion factors

#define RGAS    8.314472           // gas constant [J K-1 mol-1]
#define TEMPSTD 298.15             // standard temperature [K]
#define WAVNHRT 219474.6314        // cm-1 hartree-1
#define KJMLHRT 2625.499625        // kJ/mol hartree-1
#define BIWAA2  16.8576291         // cm-1 amu A2
#define GHZWAVN 29.9792458         // GHz (cm-1)-1
#define KELWAVN 1.438775246        // K (cm-1)-1
#define JMLWAVN 11.9626565         // J mol-1 (cm-1)-1

/* ===== I/O functions ================================================ */

//   output formatting class

class outFormat {
 public:
  ostream *ostrm;
  ios::fmtflags flagSave;
  int prec;
 public:
  outFormat();
  outFormat(ostream *os);
  ~outFormat();
  void restore();
};

void showRevision();
int getLine(istream &is, string &line);
int getLineNonBlank(istream &is, string &line);
int getLineValid(istream &is, string &line, string cmtsep);
int inpToken(istream &is, vector<string> &vs);
int inpTokenKeyValues(istream &is, string &key, vector<string> &vs);

/* ===== string manipulation function ================================= */

string strAllToUpper(string &s);
string strAllToLower(string &s);
string ltrim(string s);
string rtrim(string s);
string trim(string s);
vector<string> splitTokens(string &s, const string &sep);
string format(double v);
string format(int v);
string format(int v, int len, char filc);
string getBasePart(string flname);
string getExtPart(string flname);

#endif
