/* ======================================================================
     "libbx1CP2.h"
       - header file for "libbx1CP2.cc"
             Copyright (c) 2004-2006  by A. Miyoshi, Univ. Tokyo
 ====================================================================== */

#ifndef _LIBBX1CP2_H_
#define _LIBBX1CP2_H_

#include "libbx1com.h"
#include "lapack_dsbev/wcpp_dsbev.h"

/* ===== power series potential ======================================= */

class powerSeries : public map<int, double> {
 public:
  int maxOrder();
  powerSeries operator/(double den);
  vector<double> coefVect();
};

/* ===== associated Legendre basis and related classes ================ */

/* ----- global functions and constants ----------------- */

double intFactorial(int x);         // --- factorial of an interger
double intDblFactrl(int x);         // --- double factorial of an integer
double recK(int n, int m);          // --- Knm coefficient for NnmPnm
double nrmPnn(int n, double x);     // --- normalized Pnn(x)
vector<double> nrmPnm(int m, int nmax, double x); // --- normalized Pnm(x)

enum { OUTnone = 0, OUTvec = 1, OUTfunc = 2, OUTall = 3 };

/* ----- recurrent power expansion coefficients --------- */

class recurExpnCoef : public vector<double> {
 public:
  int mOrd;     // order 'm' of associated Legendre polynomial
  int jMax;     // maximum degree 'n' or max J
  int kCent;    // center of expansion
  int iPow;     // power degree
 public:
  recurExpnCoef();
  recurExpnCoef(int m, int jmx, int kc);
  void clear();
  void init(int m, int jmx, int kc);
  void incPow();
  void addTo(vector<double> &c, double f);
};

/* ----- eigenvalues for 2-D symmetric rotor ------------ */

class eigenVal2Dsrot : public vector<vector<double> > {
 public:
  vector<pair<int, int> > qnTable;  // quantum number table for sorting
 public:
  void clear();
  int numEgnv();
  void evalSort();
  double getEgnv(int idx);
};

/* ----- associated Legendre basis ---------------------- */

class basisAssocLegendre {
 public:
  enum { MXNBAS = 2001,  // maximum number of basis functions
         SZWLPK = 6001,  // size of DSBEV work array: max(1, 3*MXNBAS-2)
         MXRWAB = 51 };  // max nbr rows of DSBEV ab mat: max(maxOrder+1)
  int maxJdeg;                 // maximum J quantum number (degree)
  int outJmax;                 // maximum J for vector/function output
  double rotB;                 // rotational constant
  nucSpinStat nSpinW;          // nuclear spin weights
  powerSeries potE;            // cos(theta) power-series potential energy
  eigenVal2Dsrot egnVal;       // eigen values
 protected:
  int mJcur;                   // current mJ
  double ab[MXNBAS][MXRWAB];   // arrays used by DSBEV
  double z[MXNBAS][MXNBAS];
  double w[MXNBAS], work[SZWLPK];
  enum errcode { FAILopen, LRGQerr, LMTQrng };

 public:
  basisAssocLegendre();
  basisAssocLegendre(int jmx, int ojm, double rtc, nucSpinStat &nsw,
   powerSeries &pe);
  void clear();
  void init(int jmx, int ojm, double rtc, nucSpinStat &nsw,
   powerSeries &pe);
  int solveOnly();
  int solveWrite(string &basfln, int outCtrl);
  int writeEnergies(string &basfln, int maxRNE);
  int writePartFuncs(string &basfln, int maxRNE, tempList &tlist);
 protected:
  void clear_ab(int m);
  vector<double> kappa(int m, int kc);
  void set_ab_L(int m);
  int solveForOneM(int m);
  int writeVectors(string &basfln);
  int writeFunctions(string &basfln);
  int errms(errcode ec, string ms = "");
};

#endif
