/* ======================================================================
     "libbx1FS.h"
       - header file for "libbx1FS.cc"
             Copyright (c) 2004-2005  by A. Miyoshi, Univ. Tokyo
 ====================================================================== */

#ifndef _LIBBX1FS_H_
#define _LIBBX1FS_H_

#include "libbx1com.h"
#include "lapack_zhbev/wcpp_zhbev.h"

/* ===== Fourier series potential ===================================== */

class fourSeries : public map<int, double> {
 public:
  int maxAbsOrder;
 public:
  fourSeries();
  void finishSetting();
};

/* ===== free rotor basis set ========================================= */

class basisFreeRotor {
 public:
  enum { MXNBAS = 3001,  // maximum number of basis functions
         SZRWLP = 9001,  // size of ZHBEV rwork array: max(1, 3*MXNBAS-2)
         MXRWAB = 51 };  // max nbr rows of ZHBEV ab mat: maxAbsOrder+1
  int maxAbsQntNbr;          // maximum absolute quantum number
  double rotB;               // rotational constant
  fourSeries potE;           // potential energy
  nucSpinStat nSpinW;        // nuclear spin weights
                             // arrays used by ZHBEV
  complex<double> ab[MXNBAS][MXRWAB], z[MXNBAS][MXNBAS], work[MXNBAS];
  double w[MXNBAS], rwork[SZRWLP];
  complex<double> f[MXNBAS];
 public:
  basisFreeRotor();
  void clear();
  int solve(int maqn);
  complex<double> *eigenFunc(double x, int laqn);
 private:
  void setBasisFunc(double x);
};

#endif
