/* ======================================================================
     "libbx1PS.h"
       - header file for "libbx1PS.cc"
             Copyright (c) 2004-2005  by A. Miyoshi, Univ. Tokyo
 ====================================================================== */

#ifndef _LIBBX1PS_H_
#define _LIBBX1PS_H_

#include "libbx1com.h"
#include "lapack_dsbev/wcpp_dsbev.h"

/* ===== extended double variable class =============================== */

//   double variable extended for large exponent

class extDouble {
 public:
  int sign;
  double lnval;
 public:
  extDouble();
  extDouble(int s, double lv);
  extDouble(double v);
  void clear();
  void set(int s, double lv);
  void set(double v);
  double val();
  extDouble operator*(double v);
  extDouble operator*(extDouble lev);
  extDouble operator+(extDouble lev);
};

/* ===== power series potential and related classes =================== */

class powerTerm {
 public:
  int order;
  double coef;
};

class powerSeries : public vector<powerTerm> {
 public:
  int maxOrder;
 public:
  powerSeries();
  void setMaxOrder();
};

/* ===== Hermite-Gaussian basis set and related classes =============== */

//   parametrized coefficients of analytical integral for
//      Hermite-Gaussian basis and power series potential

class termIdxHGPS : public vector<char> {
 public:
  double value(int k);
};

class coefTermsHGPS : public vector<termIdxHGPS> {
 public:
  double value(int k);
};

class integCoefHGPS : public vector<map<int, coefTermsHGPS> > {
 public:
  int maxOrder;
 public:
  integCoefHGPS();
  integCoefHGPS(int mord);
  void setCoefIndices(int mord);
};

//   Hermite-Gaussian basis set

class basisHermiteGauss {
 public:
  enum { MXNBAS = 2001,  // maximum number of basis functions
         SZWLPK = 6001,  // size of DSBEV work array: max(1, 3*MXNBAS-2)
         MXRWAB = 51 };  // max nbr rows of DSBEV ab mat: max(maxOrder+1)
  int nBasis;                // number of basis functions
  double rotB;               // B or h~2/2m
  double alpha;              // coordinate scaling factor
  nucSpinStat nSpinW;        // nuclear spin weights
 protected:
  extDouble cH0, cH1;
  extDouble c[MXNBAS][2];    // coefficients for recursion relation
  integCoefHGPS cInteg;
                             // arrays used by DSBEV
  double ab[MXNBAS][MXRWAB], z[MXNBAS][MXNBAS];
  double w[MXNBAS], work[SZWLPK];
  double b[MXNBAS], f[MXNBAS];
 public:
  basisHermiteGauss();
  basisHermiteGauss(int nbas, int mord, double B, double alp, nucSpinStat &nsw);
  void init();
  void setParam(int nbas, int mord, double B, double alp, nucSpinStat &nsw);
  int numBasis();
  int setNumBasis(int nbas);
  double HGalpha();
  int solve(powerSeries &potE);
  double *getEigenValues();
  double *getEgnVecCoefs(int ibf);
  double *getEigenFunc(double y, int mqn);
 private:
  void setBasFunc(double y, int nbas);
};

#endif
