# ========================================================================
#    This is a part of "bex1d"
#                        - One-dimensional Schroedinger equation solver
#            Copyright (c) 2004-2012  by A. Miyoshi, Univ. Tokyo
#                      last edited: Mar.  12, 2012
# ========================================================================

# ----- Operating-System dependent commands and file extensions --------

RM = rm -f
#RM = del
MV = mv
#MV = move
AR = ar cr 
#AR = lib /OUT:
XO = o
#XO = obj
XA = a
#XA = lib

# ----- names of C++ and Fortran compilers and options -----------------
# ----- suggested names of C++ and Fortran compilers
#  GNU compiler (version 4)         .. g++, gfortran
#               (version 3)         .. g++, g77
#  Intel compilers for Linux/MacOS  .. icc, ifort
#                  for Windows      .. icl, ifort
# ----- suggested optimization flags
#  Intel compilers for Windows      .. -O2
#  Others                           .. -O
# ----------------------------------------------------------------------

CPCMP = g++ -O
#CPCMP = icc -O
#CPCMP = icl -O2

#F7CMP = gfortran -O
F7CMP = g77 -O
#F7CMP = ifort -O
#F7CMP = ifort -O2

# ----- names of the LAPACK/BLAS library files -------------------------
#                           (to build a part of LAPACK/BLAS)
LPKSBE = lapack_dsbev.$(XA)
LPKPOS = lapack_dposv.$(XA)
LPKHBE = lapack_zhbev.$(XA)
#                           (to use pre-build LAPACK/BLAS)
#LPKSBE = wcpp_dsbev.$(XO)
#LPKPOS = wcpp_dposv.$(XO)
#LPKHBE = wcpp_zhbev.$(XO)

# ----- program to be used for linking ---------------------------------

LNKER = $(CPCMP)
#LNKER = $(F7CMP) -nofor_main

# ----- library options to the linker ----------------------------------
#  when FLPKSE/LPKPOS != (null)
#   GNU compiler (gfortran)         .. -lgfortran
#                (g77)              .. -lg2c
#   Intel compilers for Linux       .. -cxxlib
#   Intel compilers for Windows     .. (null)
#  when FLPKSE/LPKPOS == (null)
#   BLAS/LAPACK (gfortran-built)    .. -llapack -lblas -lgfortran
#               (g77-built)         .. -llapack -lblas -lg2c
#   intel MKL                       .. (consult the MKL manual)
# ----------------------------------------------------------------------

#LLIBS = -lgfortran
LLIBS = -lg2c
#LLIBS = -cxxlib
#LLIBS =
#LLIBS = -llapack -lblas -lgfortran
#LLIBS = -llapack -lblas -lg2c
#LLIBS = -lmkl_em64t -lguide -lpthread -cxxlib
#LLIBS = -lmkl_lapack -lmkl_em64t -lguide -lpthread -cxxlib
#LLIBS = mkl_intel_c.lib mkl_intel_thread.lib mkl_core.lib libiomp5mt.lib

# ===== no need to edit below in usual circumstances =====================

LOBJLL = bx1fitPlls.$(XO) libnumLls.$(XO) libbasIOstr.$(XO) $(LPKPOS)
HEADLL = libnumLls.h libbasIOstr.h lapack_dposv/wcpp_dposv.h

LOBJPS = bx1VIBsol.$(XO) libbx1PS.$(XO) libbx1com.$(XO) libbasIOstr.$(XO) $(LPKSBE)
HEADPS = libbx1PS.h libbx1com.h libbasIOstr.h lapack_dsbev/wcpp_dsbev.h

LOBJFS = bx1HRsol.$(XO) libbx1FS.$(XO) libbx1com.$(XO) libbasIOstr.$(XO) $(LPKHBE)
HEADFS = libbx1FS.h libbx1com.h libbasIOstr.h lapack_zhbev/wcpp_zhbev.h

LOBJCP2 = bx1S2HRsol.$(XO) libbx1CP2.$(XO) libbx1com.$(XO) libbasIOstr.$(XO) $(LPKSBE)
HEADCP2 = libbx1CP2.h libbx1com.h libbasIOstr.h lapack_dsbev/wcpp_dsbev.h

all: bx1fitPlls bx1HRsol bx1VIBsol bx1S2HRsol

bx1fitPlls : $(LOBJLL)
	$(LNKER) $(LOBJLL) $(LLIBS) -o bx1fitPlls

bx1VIBsol : $(LOBJPS)
	$(LNKER) $(LOBJPS) $(LLIBS) -o bx1VIBsol

bx1HRsol : $(LOBJFS)
	$(LNKER) $(LOBJFS) $(LLIBS) -o bx1HRsol

bx1S2HRsol : $(LOBJCP2)
	$(LNKER) $(LOBJCP2) $(LLIBS) -o bx1S2HRsol

bx1fitPlls.$(XO) : bx1fitPlls.cc bx1fitPlls.h $(HEADLL)
	$(CPCMP) -c bx1fitPlls.cc

bx1VIBsol.$(XO) : bx1VIBsol.cc bx1VIBsol.h $(HEADPS)
	$(CPCMP) -c bx1VIBsol.cc

bx1HRsol.$(XO) : bx1HRsol.cc bx1HRsol.h $(HEADFS)
	$(CPCMP) -c bx1HRsol.cc

bx1S2HRsol.$(XO) : bx1S2HRsol.cc bx1S2HRsol.h $(HEADCP2)
	$(CPCMP) -c bx1S2HRsol.cc

libnumLls.$(XO) : libnumLls.cc $(HEADLL)
	$(CPCMP) -c libnumLls.cc

libbx1PS.$(XO) : libbx1PS.cc $(HEADPS)
	$(CPCMP) -c libbx1PS.cc

libbx1FS.$(XO) : libbx1FS.cc $(HEADFS)
	$(CPCMP) -c libbx1FS.cc

libbx1CP2.$(XO) : libbx1CP2.cc $(HEADCP2)
	$(CPCMP) -c libbx1CP2.cc

libbx1com.$(XO) : libbx1com.cc libbx1com.h libbasIOstr.h
	$(CPCMP) -c libbx1com.cc

libbasIOstr.$(XO) : libbasIOstr.cc libbasIOstr.h
	$(CPCMP) -c libbasIOstr.cc

lapack_dsbev.$(XA) : lapack_dsbev/*.f lapack_dsbev/wcpp_dsbev.cc lapack_dsbev/wcpp_dsbev.h
	cd lapack_dsbev && \
	$(CPCMP) -c wcpp_dsbev.cc && \
	$(F7CMP) -c *.f && \
	$(AR)../$(LPKSBE) *.$(XO) && \
	$(RM) *.$(XO) && \
	cd ..

wcpp_dsbev.$(XO) : lapack_dsbev/wcpp_dsbev.cc lapack_dsbev/wcpp_dsbev.h
	cd lapack_dsbev && \
	$(CPCMP) -c wcpp_dsbev.cc && \
	$(MV) wcpp_dsbev.$(XO) ../ && \
	cd ..

lapack_dposv.$(XA) : lapack_dposv/*.f lapack_dposv/wcpp_dposv.cc lapack_dposv/wcpp_dposv.h
	cd lapack_dposv && \
	$(CPCMP) -c wcpp_dposv.cc && \
	$(F7CMP) -c *.f && \
	$(AR)../$(LPKPOS) *.$(XO) && \
	$(RM) *.$(XO) && \
	cd ..

wcpp_dposv.$(XO) : lapack_dposv/wcpp_dposv.cc lapack_dposv/wcpp_dposv.h
	cd lapack_dposv && \
	$(CPCMP) -c wcpp_dposv.cc && \
	$(MV) wcpp_dposv.$(XO) ../ && \
	cd ..

lapack_zhbev.$(XA) : lapack_zhbev/*.f lapack_zhbev/wcpp_zhbev.cc lapack_zhbev/wcpp_zhbev.h
	cd lapack_zhbev && \
	$(CPCMP) -c wcpp_zhbev.cc && \
	$(F7CMP) -c *.f && \
	$(AR)../$(LPKHBE) *.$(XO) && \
	$(RM) *.$(XO) && \
	cd ..

wcpp_zhbev.$(XO) : lapack_zhbev/wcpp_zhbev.cc lapack_zhbev/wcpp_zhbev.h
	cd lapack_zhbev && \
	$(CPCMP) -c wcpp_zhbev.cc && \
	$(MV) wcpp_zhbev.$(XO) ../ && \
	cd ..

clean:
	$(RM) *.$(XO) *.$(XA)

